@@ -275,7 +275,7 @@ static void r300_init_states(struct pipe_context *pipe)
 
     pipe->set_blend_color(pipe, &bc);
     pipe->set_clip_state(pipe, &cs);
-    pipe->set_scissor_state(pipe, &ss);
+    pipe->set_scissor_states(pipe, 0, 1, &ss);
     pipe->set_sample_mask(pipe, ~0);
 
     /* Initialize the GPU flush. */
@@ -379,7 +379,7 @@ struct pipe_context* r300_create_context(struct pipe_screen* screen,
                      sizeof(struct pipe_transfer), 64,
                      UTIL_SLAB_SINGLETHREADED);
 
-    r300->cs = rws->cs_create(rws, RING_GFX);
+    r300->cs = rws->cs_create(rws, RING_GFX, NULL);
     if (r300->cs == NULL)
         goto fail;
 
@@ -412,10 +412,8 @@ struct pipe_context* r300_create_context(struct pipe_screen* screen,
     r300->context.create_video_decoder = vl_create_decoder;
     r300->context.create_video_buffer = vl_video_buffer_create;
 
-    if (r300screen->caps.has_tcl) {
-        r300->uploader = u_upload_create(&r300->context, 256 * 1024, 4,
-                                         PIPE_BIND_INDEX_BUFFER);
-    }
+    r300->uploader = u_upload_create(&r300->context, 256 * 1024, 4,
+                                     PIPE_BIND_CUSTOM);
 
     r300->blitter = util_blitter_create(&r300->context);
     if (r300->blitter == NULL)
